<?php
/**
 * Created for Schaffen IT.
 * User: Thiago Traczykowski
 */

namespace Schaffen\ServiceOrder\ServiceOrder\Services;


class GenerateInvoiceFromServiceOrder
{

   /**
    * Realiza o faturamento a partir do código da nota fiscal de serviço
    *
    * @param type $codigoOs
    * @param type $nfsCodigo
    * @return type
    * @throws SCHException
    */
   public static function faturarNfs($codigoOs, $nfsCodigo)
   {

      $os = Os::find($codigoOs);
      if (!$os) {
         throw new SCHException("A OS/Orçamento código {$codigoOs} não foi encontrada.");
      }

      $nfs = NotaFiscalServico::find($nfsCodigo);

      if (!$nfs) {
         throw new SCHException("A nota fiscal de serviço código {$nfsCodigo} não foi encontrada.");
      }
      $nfs->nfs_bloqueio_modulo = 'ORDEM_SERVICO';
      $nfs->save();

      /* Cria um faturamento para relacionar as mvfs das parcelas com o faturamento */
      $faturamento = new Faturamento();
      $faturamento->fat_fk_pes_codigo = $os->os_fk_pes_codigo;
      $faturamento->fat_fk_fil_codigo = $os->os_fk_fil_codigo;
      $faturamento->save();

      //registra o log
      $osLog = new OsLog([
         "osl_fk_os_codigo" => $os->os_codigo,
         "osl_fk_too_codigo" => TipoOcorrenciaOs::FISCAL,
         "osl_ocorrencia" => "Código do faturamento: {$faturamento->fat_codigo}<br/><b>Nota fiscal de serviço código:</b>{$nfsCodigo}",
         "osl_codigo_operacao" => $nfsCodigo
      ]);
      $osLog->save();
      $nfsfnf = FaturamentoNotaFiscal::where('fnf_fk_os_codigo', $codigoOs)->where('fnf_fk_nfs_codigo', $nfsCodigo)->first();
      if (empty($nfsfnf)) {
         $faturamentoNfs = new FaturamentoNotaFiscal([
            "fnf_fk_nfs_codigo" => $nfsCodigo,
            "fnf_fk_fat_codigo" => $faturamento->fat_codigo,
            "fnf_fk_os_codigo" => $codigoOs,
         ]);
         $faturamentoNfs->save();
      }
      $faturamentoOs = new FaturamentoOs([
         "fao_fk_os_codigo" => $os->os_codigo,
         "fao_fk_fat_codigo" => $faturamento->fat_codigo
      ]);
      $faturamentoOs->save();

      //faz o vinculo dos itens da locação
      $osLocacao = OsLocacao::where("olo_fk_os_codigo", $os->os_codigo)
         ->whereNotNull("olo_data_devolucao")->get();
      if (!empty($osLocacao)) {

         foreach ($osLocacao as $locacao) {
            if ($locacao->olo_renovado == 1) {
               $locacaoReno = OsLocacaoRenovacao::where('olr_fk_olo_codigo', $locacao->olo_codigo)->first();
               $locacaoReno->olr_fk_nfs_codigo = $nfsCodigo;
               $locacaoReno->save();
            } else {
               $locacao->olo_fk_nfs_codigo = $nfsCodigo;
               $locacao->save();
            }
         }


      }


      //vincula os serviços com o faturamento
      $osProdutosServicos = OsProdutosServicos::where("osps_fk_os_codigo", $os->os_codigo)
         ->whereNull("osps_fk_fat_codigo")->where("osps_servico", 1)->get();
      foreach ($osProdutosServicos as $servico) {
         $servico->osps_fk_fat_codigo = $faturamento->fat_codigo;
         $servico->save();
      }
      return $faturamento->fat_codigo;
   }

}
